@@ -8,26 +8,7 @@ Usage:
   exo [--help] [options] <command> [<args> ...]
 
 Commands:
-  read
-  write
-  record
-  create
-  update
-  map
-  unmap
-  lookup
-  drop
-  listing
-  info
-  flush
-  usage
-  tree
-  script
-  spark
-  copy
-  diff
-  data
-
+  {{ command_list }}
 Options:
   --host=<host>        OneP URL. Default is $EXO_HOST or m2.exosite.com
   --port=<port>        OneP port. Default is $EXO_HOST or 80
@@ -57,6 +38,7 @@ from operator import itemgetter
 import logging
 from collections import defaultdict
 import StringIO
+import pytz
 # python 2.6 support
 try:
     from collections import OrderedDict
@@ -68,13 +50,18 @@ import math
 from docopt import docopt
 from dateutil import parser
 import requests
+import yaml
 
 from pyonep import onep
 import pyonep
+import timezone
 try:
     from ..exoline import __version__
+    from ..exoline.exocommon import ExoException
 except:
     from exoline import __version__
+    from exocommon import ExoException
+
 
 DEFAULT_HOST = 'm2.exosite.com'
 DEFAULT_PORT = '80'
@@ -90,6 +77,7 @@ Options:
     --limit=<limit>          number of data points to read [default: 1]
     --start=<time>
     --end=<time>             start and end times (see details below)
+    --tz=<TZ>                Olson TZ name
     --sort=<order>           asc or desc [default: desc]
     --selection=all|autowindow|givenwindow  downsample method [default: all]
     --format=csv|raw         output format [default: csv]
@@ -103,6 +91,7 @@ Options:
     {{ helpoption }}
 
     If <rid> is omitted, reads all datasources and datarules under <cik>.
+    All output is in UTC.
 
     {{ startend }}''',
     'write':
@@ -136,8 +125,11 @@ Details:
 
     If - is not present, creates a resource with common defaults.''',
     'listing':
-        '''List a client's children based on their type.\n\nUsage:
-    exo [options] listing <cik> (--type=client|dataport|datarule|dispatch) ...
+        '''List the RIDs of a client's children.\n\nUsage:
+    exo [options] listing <cik> [(--type=client|dataport|datarule|dispatch) ...]
+
+    If --type is omitted, it defaults to:
+    --type=client --type=dataport --type=datarule --type=dispatch
 
 Options:
     --plain   show only the child RIDs
@@ -212,6 +204,8 @@ Options:
 
     --counts       show item counts (from info.storage.counts)
     --level=<num>  depth to traverse, omit or -1 for no limit [default: -1]''',
+    'ut': '''Display a tree as fast as possible\n\nUsage:
+    exo [options] ut <cik>''',
     'script': '''Upload a client's a Lua script\n\nUsage:
     exo [options] script <script-file> <cik> ...
 
@@ -251,12 +245,11 @@ Options:
     'data': '''Read or write with the HTTP Data API.\n\nUsage:
     exo [options] data <cik> [--write=<alias,value> ...] [--read=<alias> ...]
 
-    Writes and/or reads a set of dataports.
     If only --write arguments are specified, the call is a write.
     If only --read arguments are specified, the call is a read.
     If both --write and --read arguments are specified, the hybrid
         write/read API is used. Writes are executed before reads.'''
-}
+    }
 
 # shared sections of documentation
 doc_replace = {
@@ -264,13 +257,37 @@ doc_replace = {
 
     2011-10-23T08:00:00-07:00
     10/1/2012
+    "2012-10-23 14:01 UTC"
     "2012-10-23 14:01"
 
+    If timezone information is omitted, local timezone is assumed
     If time part is omitted, it assumes 00:00:00.
     To report through the present time, omit --end or pass --end=now''',
     '{{ helpoption }}': '''-h --help  Show this screen.''',
 }
 
+# load plugins
+import importlib
+import plugins as plugins_package
+#plugin_path = os.path.dirname(os.path.realpath(plugins_package.__file__))
+plugins = []
+for module_name in plugins_package.__all__:
+    #plugin = imp.load_module(module_name, *imp.find_module(module_name, [plugin_path]))
+
+    try:
+        plugin = importlib.import_module('plugins.' + module_name)
+    except:
+        plugin = importlib.import_module('exoline.plugins.' + module_name, package='test')
+
+    # instantiate plugin
+    p = plugin.Plugin()
+    plugins.append(p)
+
+    # get documentation
+    cmd_doc[p.command()] = plugin.__doc__
+
+
+# perform substitutions on command documentation
 for k in cmd_doc:
     # helpoption is appended to any commands that don't already have it
     if '{{ helpoption }}' not in cmd_doc[k]:
@@ -279,12 +296,6 @@ for k in cmd_doc:
         cmd_doc[k] = cmd_doc[k].replace(r, doc_replace[r])
 
 
-class ExoException(Exception):
-    pass
-
-
-class RPCException(Exception):
-    pass
 
 
 class ExolineOnepV1(onep.OnepV1):
@@ -297,19 +308,24 @@ class ExoRPC():
     '''Wrapper for pyonep RPC API.
     Raises exceptions on error and provides some reasonable defaults.'''
     regex_rid = re.compile("[0-9a-zA-Z]{40}")
+
+    class RPCException(Exception):
+        pass
+
     def __init__(self,
-            host=DEFAULT_HOST,
-            port=None,
-            httptimeout=60,
-            https=False,
-            verbose=True):
+                 host=DEFAULT_HOST,
+                 port=None,
+                 httptimeout=60,
+                 https=False,
+                 verbose=True):
 
         if port is None:
             port = DEFAULT_PORT_HTTPS if https else DEFAULT_PORT
         self.exo = ExolineOnepV1(host=host,
                                port=port,
                                httptimeout=httptimeout,
-                               https=https)
+                               https=https,
+                               agent="Exoline {0}".format(__version__))
 
     def _raise_for_response(self, isok, response, call=None):
         if not isok:
@@ -317,7 +333,7 @@ class ExoRPC():
                 msg = str(response)
             else:
                 msg = '{0} ({1})'.format(str(response), str(call))
-            raise RPCException(msg)
+            raise ExoRPC.RPCException(msg)
 
     def _raise_for_response_record(self, isok, response):
         '''Undocumented RPC behavior-- if record timestamps are invalid, isok
@@ -326,7 +342,7 @@ class ExoRPC():
         self._raise_for_response(isok, response)
         if type(response) is list:
             # TODO: does this always indicate an error condition?
-            raise RPCException(', '.join(['{0}: {1}'.format(msg, t) for msg, t in response]))
+            raise ExoRPC.RPCException(', '.join(['{0}: {1}'.format(msg, t) for msg, t in response]))
 
     def _raise_for_deferred(self, responses):
         r = []
@@ -368,7 +384,6 @@ class ExoRPC():
             # if cik doesn't look like a cik, maybe it's a shortcut
             configfile = os.path.join(os.environ['HOME'], '.exoline')
             try:
-                import yaml
                 with open(configfile) as f:
                     config = yaml.safe_load(f)
                     if 'keys' in config:
@@ -553,6 +568,9 @@ class ExoRPC():
         return response
 
     def create_dataport(self, cik, format, name=None):
+        '''Create a dataport child of cik with common defaults.
+           (retention count duration set to "infinity"). Returns
+           RID string of the created dataport.'''
         desc = {"format": format,
                 "retention": {
                     "count": "infinity",
@@ -563,23 +581,25 @@ class ExoRPC():
         return self.create(cik, 'dataport', desc)
 
     def create_client(self, cik, name=None, desc=None):
+        '''Create a client child of cik with common defaults.
+        ('inherit' set for all limits). Returns RID string
+        of the created client.'''
         if desc is None:
             # default description
-            desc = {'limits': {
-                              'client': 'inherit',
-                              'dataport': 'inherit',
-                              'datarule': 'inherit',
-                              'disk': 'inherit',
-                              'dispatch': 'inherit',
-                              'email': 'inherit',
-                              'email_bucket': 'inherit',
-                              'http': 'inherit',
-                              'http_bucket': 'inherit',
-                              'share': 'inherit',
-                              'sms': 'inherit',
-                              'sms_bucket': 'inherit',
-                              'xmpp': 'inherit',
-                              'xmpp_bucket': 'inherit'},
+            desc = {'limits': {'client': 'inherit',
+                               'dataport': 'inherit',
+                               'datarule': 'inherit',
+                               'disk': 'inherit',
+                               'dispatch': 'inherit',
+                               'email': 'inherit',
+                               'email_bucket': 'inherit',
+                               'http': 'inherit',
+                               'http_bucket': 'inherit',
+                               'share': 'inherit',
+                               'sms': 'inherit',
+                               'sms_bucket': 'inherit',
+                               'xmpp': 'inherit',
+                               'xmpp_bucket': 'inherit'},
                     'writeinterval': 'inherit'}
         if name is not None:
             desc['name'] = name
@@ -593,11 +613,13 @@ class ExoRPC():
             self._raise_for_deferred(self.exo.send_deferred(cik))
 
     def map(self, cik, rid, alias):
+        '''Creates an alias for rid. '''
         isok, response = self.exo.map(cik, rid, alias)
         self._raise_for_response(isok, response)
         return response
 
     def unmap(self, cik, alias):
+        '''Removes an alias a child of calling client.'''
         isok, response = self.exo.unmap(cik, alias)
         self._raise_for_response(isok, response)
         return response
@@ -650,6 +672,7 @@ class ExoRPC():
              options={},
              cikonly=False,
              recursive=False):
+        '''Returns info for RID as a dict.'''
         if recursive:
             rid = None if type(rid) is dict else rid
             response = self._infotree(cik, rid=rid, options=options)
@@ -681,7 +704,7 @@ class ExoRPC():
             print("{0}:{1} {2}".format(
                   metrics[i], ' ' * (maxlen - len(metrics[i])), r))
 
-    def _print_node(self, rid, info, aliases, cli_args, spacer, islast):
+    def _print_node(self, rid, info, aliases, cli_args, spacer, islast, max_name):
         typ = info['basic']['type']
         if typ == 'client':
             id = 'cik: ' + info['key']
@@ -709,10 +732,6 @@ class ExoRPC():
         def add_opt(o, label, value):
             if o is True or (o in cli_args and cli_args[o] is True):
                 opt[label] = value
-        add_opt(True, typ + ' name', name)
-        if 'format' in info['description']:
-            add_opt(True, 'format', info['description']['format'])
-
         try:
             # show portals metadata if present
             # http://developers.exosite.com/display/POR/Developing+for+Portals
@@ -740,9 +759,16 @@ class ExoRPC():
         if 'storage' in info and 'count' in info['storage']:
             add_opt(True, 'count', info['storage']['count'])
 
-        print(u'{0}{1} {2}'.format(
+        if 'format' in info['description']:
+            desc = info['description']['format'] + ' ' + typ + ' ' + id
+        else:
+            desc = typ + ' ' + id
+
+        print(u'{0}{1}{2} {3} {4}'.format(
             spacer,
-            id,
+            name,
+            ' ' * (max_name - len(name)),
+            desc,
             u'' if len(opt) == 0 else u'({0})'.format(u', '.join(
                 [u'{0}: {1}'.format(k, v) for k, v in opt.iteritems()]))))
 
@@ -758,6 +784,7 @@ class ExoRPC():
                 exclude.append('counts')
                 exclude.append('storage')
             info_options = self.make_info_options(exclude=exclude)
+            #pprint(info_options)
             # todo: combine these (maybe also the _listing_with_info function
             # below?)
             rid = self.lookup(cik, "")
@@ -772,7 +799,8 @@ class ExoRPC():
                              root_aliases,
                              cli_args,
                              spacer,
-                             islast=True)
+                             True,
+                             len(info['description']['name']))
             if max_level == 0:
                 return
             level += 1
@@ -786,9 +814,13 @@ class ExoRPC():
         except pyonep.exceptions.OnePlatformException:
             print(spacer + u"  └─listing for {0} failed. Is info['basic']['status'] == 'expired'?".format(cik))
         else:
+            # calculate the maximum length name of all children
+            lengths = [len(l[1]['description']['name']) for i in range(len(types)) for l in listing[i].iteritems()]
+            max_name = 0 if len(lengths) == 0 else max(lengths)
+
             # print everything
             for t_idx, t in enumerate(types):
-                typelisting = listing[t_idx]
+                typelisting = OrderedDict(sorted(listing[t_idx].iteritems(), key=lambda x: x[1]['description']['name'].lower()))
                 islast_nonempty_type = (t_idx == len(types) - 1) or (all(len(x) == 0 for x in listing[t_idx + 1:]))
                 for rid_idx, rid in enumerate(typelisting):
                     info = typelisting[rid]
@@ -803,11 +835,142 @@ class ExoRPC():
 
                     if t == 'client':
                         next_cik = info['key']
-                        self._print_node(rid, info, aliases, cli_args, own_spacer, islast)
+                        self._print_node(rid, info, aliases, cli_args, own_spacer, islast, max_name)
                         if max_level == -1 or level < max_level:
                             self.tree(next_cik, info['aliases'], cli_args, child_spacer, level=level + 1, info_options=info_options)
                     else:
-                        self._print_node(rid, info, aliases, cli_args, own_spacer, islast)
+                        self._print_node(rid, info, aliases, cli_args, own_spacer, islast, max_name)
+
+    def _uberlistingtree(self, cik, rid=None, stats={'calls': 0}):
+        '''Get something like this: {'rid': '<rid1>',
+                                     'type': 'client',
+                                     'children': [{'rid': '<rid2>',
+                                                   'type': 'dataport'}]}'''
+        types = ['client', 'dataport', 'datarule', 'dispatch']
+        # result tree
+        tree = {}
+        # generation 0 (self)
+        if rid is None:
+            rid, listing = self._exomult(cik, [['lookup', 'aliased', ''],
+                                               ['listing', types]])
+            stats['calls'] += 1
+        else:
+            # connect_as() is an awkward API. What advantage does it have
+            # over passing in an auth dict?
+            self.exo.connect_as(rid)
+            listing = self._exomult(cik, [['listing', types]])[0]
+            stats['calls'] += 1
+            self.exo.connect_as(None)
+
+        tree['rid'] = rid
+
+        # children should look like this:
+        # [{'type': 'client',
+        #   'rid': '<rid1>'},
+        #  {'type': 'dataport',
+        #   'rid': '<rid2>'}]
+        def children(listing):
+            return [{'type': type, 'rid': rid} for i, type in enumerate(types) for rid in listing[i]]
+        tree['children'] = children(listing)
+
+        for node in tree['children']:
+            if node['type'] == 'client':
+                subtree = self._uberlistingtree(cik, node['rid'], stats)
+                if 'children' not in node:
+                    node['children'] = []
+                node['children'] = subtree['children']
+
+        return tree
+
+
+    def _uberinfotree(self, cik):
+        '''Get all info for a cik and its children in a nested dict.
+        The basic unit is {'rid': '<rid>',
+                           'info': <info>,
+                           'children': {'client': [child1],
+                                        'dataport': [child2]}
+                                        }
+        where <info-with-children> is just the info object for that node
+        with the addition of 'children' key, which is a dict containing
+        more nodes. Here's an example return value:
+
+           {'rid': '<rid 0>',
+            'info': {'description': ...,
+                     'basic': ....,
+                     ...},
+            'children: [{'rid': '<rid 1>',
+                         'info': {'description': ...,
+                                  'basic': ...},
+                         'children': [{'rid': '<rid 2>',
+                                       'info': {'description': ...,
+                                                'basic': ...},
+                                       'children: [] } } },
+                                      {'rid': '<rid 3>',
+                                       'info': {'description': ...,
+                                                'basic': ...},
+                                       'children': {} } }] } }
+
+           As it's building this nested dict, it calls nodeidfn with the rid and info
+           (w/o children) for each node.
+        '''
+        types = ['client', 'dataport', 'datarule', 'dispatch']
+
+        # result tree
+        tree = {}
+
+        # list of result nodes to fill out in the next generation
+        nextgen = []
+
+        # generation 0 (self)
+        rid, listing = self._exomult(cik, [['lookup', 'aliased', ''],
+                                           ['listing', types]])
+
+        tree['rid'] = rid
+
+        # children should look like this:
+        # [{'type': 'client',
+        #   'rid': '<rid1>'},
+        #  {'type': 'dataport',
+        #   'rid': '<rid2>'}]
+        def children(listing):
+            return [{'type': type, 'rid': rid} for i, type in enumerate(types) for rid in listing[i]]
+        tree['children'] = children(listing)
+
+        # calculate next generation list
+        # TODO: paging
+        nextgen = tree['children']
+
+        while len(nextgen) > 0:
+            print(len(nextgen))
+            curgen = nextgen
+            nextgen = []
+
+            cmds = []
+            for node in curgen:
+                if node['type'] == 'client':
+                    cmds.append(['listing', types])
+
+            if len(cmds) > 0:
+                result = self._exomult(cik, cmds)
+            else:
+                break
+
+            result_idx = 0
+            for i, node in enumerate(curgen):
+                # TODO: add results to members of curgen, and update nextgen
+                if node['type'] == 'client':
+                    r = result[result_idx]
+                    node['children'] = children(r)
+                    result_idx += 1
+                    nextgen.extend(node['children'])
+
+        return tree
+
+    def ubertree(self, cik):
+        #pprint(self._uberinfotree(cik)) #, options={'key': True, 'basic': True}))
+        stats = {'calls': 0}
+        pprint(self._uberlistingtree(cik, rid=None, stats=stats)) #, options={'key': True, 'basic': True}))
+        print('calls: {0}'.format(stats['calls']))
 
     def drop_all_children(self, cik):
         isok, listing = self.exo.listing(cik,
@@ -881,14 +1044,15 @@ class ExoRPC():
                       filename,
                       name=None,
                       recursive=False,
-                      create=False):
+                      create=False,
+                      filterfn=lambda script: script):
         try:
             f = open(filename)
         except IOError:
             raise ExoException('Error opening file {0}.'.format(filename))
         else:
             with f:
-                text = f.read().strip()
+                text = filterfn(f.read())
                 if name is None:
                     # if no name is specified, use the file name as a name
                     name = os.path.basename(filename)
@@ -898,7 +1062,10 @@ class ExoRPC():
                         if rid is not None or create:
                             self._upload_script(cik, name, text, rid=rid)
                         else:
-                            print("Skipping CIK: {0} -- {1} not found (--create will create it)".format(cik, name))
+                            print("Skipping CIK: {0} -- {1} not found".format(cik, name))
+                            if not create:
+                                    print('Pass --create to create it')
+
 
                     if recursive:
                         self.cik_recursive(cik, up)
@@ -987,7 +1154,7 @@ class ExoRPC():
 
         # check counts
         counts = self._counttypes(infotree)
-        destinfo = self.info(destcik)
+        destinfo = self.info(destcik, options={'description': True, 'counts': True})
 
         noroom = ''
         for typ in counts:
@@ -1247,7 +1414,10 @@ class ExoData():
 
 
 def parse_ts(s):
-    return int(time.mktime(parser.parse(s).timetuple()))
+    return parse_ts_tuple(parser.parse(s).timetuple())
+
+def parse_ts_tuple(t):
+    return int(time.mktime(t))
 
 def is_ts(s):
     return re.match('^[0-9]+$', s) is not None
@@ -1263,7 +1433,7 @@ def get_startend(args):
     else:
         start = parse_ts(start)
     if end is None or end == 'now':
-        end = int(time.mktime(datetime.now().timetuple()))
+        end = int(parse_ts_tuple(datetime.now().timetuple()))
     elif is_ts(end):
         end = int(end)
     else:
@@ -1292,20 +1462,7 @@ def spark(numbers, empty_val=None):
     When value is empty_val, show no bar.
 
     https://github.com/1stvamp/py-sparkblocks
-        start = args['--start']
-        end = args['--end']
-        parse_ts = lambda(s): int(time.mktime(parser.parse(s).timetuple()))
-        is_ts = lambda(s): re.match('^[0-9]+$', s) is not None
-        if is_ts(start):
-            start = int(start)
-        else:
-            start = parse_ts(start)
-        if end is None or end == 'now':
-            end = int(time.mktime(datetime.now().timetuple()))
-        elif is_ts(end):
-            end = int(end)
-        else:
-            end = parse_ts(end)
+
     Based on:
       https://github.com/holman/spark
     and:
@@ -1432,6 +1589,21 @@ def read_cmd(er, cik, rids, args):
 
     fmt = args['--format']
 
+    tz = args['--tz']
+
+    if tz == None:
+        # default to UTC
+        tz = timezone.localtz()
+    else:
+        try:
+            tz = pytz.timezone(tz)
+        except Exception, e:
+            #default to utc if error
+            print "Error parsing --tz option, defaulting to local timezone"
+            tz = timezone.localtz()
+
+    recarriage = re.compile('\r(?!\\n)')
+
     def printline(timestamp, val):
         if fmt == 'raw':
             print(val[0])
@@ -1439,12 +1611,21 @@ def read_cmd(er, cik, rids, args):
             if timeformat == 'unix':
                 dt = timestamp
             elif timeformat == 'iso8601':
-                dt = datetime.isoformat(datetime.fromtimestamp(timestamp))
+                dt = datetime.isoformat(pytz.utc.localize(datetime.utcfromtimestamp(timestamp)))
             else:
-                dt = datetime.fromtimestamp(timestamp)
+                dt = tz.localize(datetime.utcfromtimestamp(timestamp))
 
             row = {'timestamp': str(dt)}
-            values = dict([(str(headers[i + 1]), val[i]) for i in range(len(rids))])
+
+            def stripcarriage(s):
+                # strip carriage returns not followed
+                if type(s) is str:
+                    return recarriage.sub('', s)
+                else:
+                    return s
+
+            values = dict([(str(headers[i + 1]), stripcarriage(val[i])) for i in range(len(rids))])
+
             row.update(values)
             dw.writerow(row)
 
@@ -1489,13 +1670,14 @@ def read_cmd(er, cik, rids, args):
         if chunkhours is not None and (start is None or end is None):
             raise ExoException(
                 "--chunkhours requires --start and --end be set")
-        for t, v in er.readmult(cik,
+        result = er.readmult(cik,
                                 rids,
                                 sort=args['--sort'],
                                 starttime=start,
                                 endtime=end,
                                 limit=limit,
-                                chunkhours=chunkhours):
+                                chunkhours=chunkhours)
+        for t, v in result:
             printline(t, v)
 
 
@@ -1526,7 +1708,6 @@ def handle_args(cmd, args):
     else:
         # for data ip command
         cik = None
-
     def rid_or_alias(rid):
         '''Translate what was passed for <rid> to an alias object if
            it doesn't look like a RID.'''
@@ -1670,6 +1851,8 @@ def handle_args(cmd, args):
             er.drop(cik, rids)
     elif cmd == 'listing':
         types = args['--type']
+        if len(types) == 0:
+            types = ['client', 'dataport', 'datarule', 'dispatch']
         listing = er.listing(cik, types)
         if args['--plain']:
             for l in listing:
@@ -1713,6 +1896,8 @@ def handle_args(cmd, args):
     # special commands
     elif cmd == 'tree':
         er.tree(cik, cli_args=args)
+    elif cmd == 'ut':
+        er.ubertree(cik)
     elif cmd == 'script':
         # cik is a list of ciks
         er.upload_script(cik, args['<script-file>'],
@@ -1721,8 +1906,8 @@ def handle_args(cmd, args):
                          create=args['--create'])
     elif cmd == 'spark':
         days = int(args['--days'])
-        end = time.mktime(datetime.now().timetuple())
-        start = time.mktime((datetime.now() - timedelta(days=days)).timetuple())
+        end = parse_ts_tuple(datetime.now().timetuple())
+        start = parse_ts_tuple((datetime.now() - timedelta(days=days)).timetuple())
         numstd = args['--stddev']
         numstd = int(numstd) if numstd is not None else None
         show_intervals(er, cik, rids[0], start, end, limit=1000000, numstd=numstd)
@@ -1770,7 +1955,21 @@ def handle_args(cmd, args):
             alias_values = get_alias_values(writes)
             ed.write(cik, alias_values)
     else:
-        raise ExoException("Command not handled")
+        # search plugins
+        handled = False
+        for plugin in plugins:
+            if cmd in plugin.command():
+                options = {'cik': cik, 'rpc': er, 'exception': ExoException}
+                try:
+                    options['data'] = ed
+                except NameError:
+                    # no problem
+                    pass
+                plugin.run(cmd, args, options)
+                handled = True
+                break
+        if not handled:
+            raise ExoException("Command not handled")
 
 
 class DiscreetFilter(object):
@@ -1798,8 +1997,10 @@ def cmd(argv=None, stdin=None, stdout=None, stderr=None):
     if stdout is not None:
         sys.stdout = stdout
 
+    command_list = '  '.join([k + '\n' for k in cmd_doc])
+    doc = __doc__.replace('{{ command_list }}', command_list)
     args = docopt(
-        __doc__,
+        doc,
         version="Exosite Command Line {0}".format(__version__),
         options_first=True)
 
@@ -1813,7 +2014,6 @@ def cmd(argv=None, stdin=None, stdout=None, stderr=None):
         return 1
     # merge command-specific arguments into general arguments
     args.update(args_cmd)
-
     # turn on stdout/stderr filtering
     if args['--discreet']:
         sys.stdout = DiscreetFilter(sys.stdout)
@@ -1838,7 +2038,7 @@ def cmd(argv=None, stdin=None, stdout=None, stderr=None):
         # command line tool threw an exception on purpose
         sys.stderr.write("Command line error: {0}\r\n".format(ex))
         return 1
-    except RPCException as ex:
+    except ExoRPC.RPCException as ex:
         # pyonep library call signaled an error in return values
         sys.stderr.write("One Platform error: {0}\r\n".format(ex))
         return 1
